##
## Author......: Luis Gustavo Nardin
## Last Change.: 08/05/2016
##
library(data.table)
library(ggplot2)
library(grid)
library(gridExtra)
library(doParallel)
registerDoParallel(cores=2)


###############
## PATHS
###############
outputDir <- "../figures"

###############
## FUNCTIONS
###############
source("calcExpectedTime.R")


###############
## DISEASE 1 INPUT PARAMETERS
###############
# Disease duration
duration <- 65

# R0
R0 <- 2

# gamma
gamma <- 1 / duration

# beta
betaS <- R0 / duration

# Infection probability (Susceptible)
bs <- 1 - exp(-betaS)

# Prophylactic protection
rho <- 0.50

# Recover probability
g <- 1 - exp(-gamma)

# Discount factor (0 = No discount)
lambda <- 0

# Fear factor (1 = No fear)
kappa <- 1

# Payoffs (S, P, I, R)
payoffs <- c(1, 0.95, 0.10, 0.95)

# Disease's prevalence
i <- 0.5

# Planning Horizon
H <- seq(1, 365)

eTime <- foreach(h=H, .combine=rbind) %dopar%
  calc_expectedTime(h, i, bs, rho, g, lambda, kappa, payoffs)

total <- eTime[,6] + eTime[,7] + eTime[,8]
dataS <- cbind(eTime[,1], eTime[,2], eTime[,3], eTime[,4], eTime[,5], "S", eTime[,6] / total)
dataS <- rbind(dataS, cbind(eTime[,1], eTime[,2], eTime[,3], eTime[,4], eTime[,5], "I", eTime[,7] / total))
dataS <- rbind(dataS, cbind(eTime[,1], eTime[,2], eTime[,3], eTime[,4], eTime[,5], "R", eTime[,8] / total))
dataS <- rbind(dataS, cbind(eTime[,1], eTime[,2], eTime[,3], eTime[,4], eTime[,5], "U", eTime[,9]))

total <- eTime[,10] + eTime[,11] + eTime[,12]
dataP <- cbind(eTime[,1], eTime[,2], eTime[,3], eTime[,4], eTime[,5], "P", eTime[,10] / total)
dataP <- rbind(dataP, cbind(eTime[,1], eTime[,2], eTime[,3], eTime[,4], eTime[,5], "I", eTime[,11] / total))
dataP <- rbind(dataP, cbind(eTime[,1], eTime[,2], eTime[,3], eTime[,4], eTime[,5], "R", eTime[,12] / total))
dataP <- rbind(dataP, cbind(eTime[,1], eTime[,2], eTime[,3], eTime[,4], eTime[,5], "U", eTime[,13]))

dataS <- data.table(dataS)
colnames(dataS) <- c("h", "i", "rho", "lambda", "kappa", "type", "value")
dataS[which(value < 0)] <- 0

dataP <- data.table(dataP)
colnames(dataP) <- c("h", "i", "rho", "lambda", "kappa", "type", "value")
dataP[which(value < 0)] <- 0

plS <- ggplot(dataS[which(type != "U")], aes(x=as.numeric(as.character(h)),
                                             y=as.numeric(as.character(value)),
                                             fill=as.factor(type),
                                             color=as.factor(type),
                                             group=as.factor(type))) +
  geom_area(position="stack") +
  #xlab(expression(paste("Planning Horizon (H)"))) +
  ylab(expression("Expected % of H")) +
  xlab("") +
  scale_y_continuous(breaks=c(0, 0.25, 0.5, 0.75, 1),
                     labels=c("0%", "25%", "50%", "75%", "100%"),
                     limits=c(-0.001, 1.001)) +
  scale_color_manual(name="",
                     values=c("S" = "black", "I" = "red2", "R" = "gold1")) +
  scale_fill_manual(name="",
                    values=c("S" = "black", "I" = "red2", "R" = "gold1")) +
  annotate("text", x=50, y=0.3, label="S", fontface="bold", size=14, color="white") +
  annotate("text", x=160, y=0.5, label="I", fontface="bold", size=14, color="white") +
  annotate("text", x=270, y=0.75, label="R", fontface="bold", size=14, color="white") +
  theme(axis.title.x = element_text(colour = 'black', size = 32, face = 'bold',
                                    margin=margin(t=0.2, unit = "cm")),
        axis.title.y = element_text(colour = 'black', size = 28, face = 'bold',
                                    margin=margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour = 'black', size = 20, face = 'bold'),
        axis.text.y = element_text(colour = 'black', size = 20, face = 'bold'),
        axis.line = element_line(colour = 'black', size = 1.5, linetype = 'solid'),
        panel.background = element_rect(fill = "transparent", colour = NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(1.0, 0, 0.0, 0.0), "cm"),
        legend.position = "none",
        legend.title = element_text(colour="black", size=14, face="bold"),
        legend.text = element_text(colour="black", size=12, face="bold"),
        legend.background = element_rect(fill="white"),
        legend.key = element_rect(fill = "white"))


plS <- plS + annotation_custom(
  grob = textGrob(label = "A", hjust = 0,
                  gp = gpar(cex = 4, fontface="bold")),
  ymin = 1.07,
  ymax = 1.07,
  xmin = -70,
  xmax = -70)

pl1 <- ggplot_gtable(ggplot_build(plS))
pl1$layout$clip[pl1$layout$name == "panel"] <- "off"


plP <- ggplot(dataP[which(type != "U")], aes(x=as.numeric(as.character(h)),
                                             y=as.numeric(as.character(value)),
                                             fill=as.factor(type),
                                             color=as.factor(type),
                                             group=as.factor(type))) +
  geom_area(position="stack") +
  #ylab(expression("Expected % of H")) +
  ylab("") +
  xlab("") +
  scale_y_continuous(breaks=c(0, 0.25, 0.5, 0.75, 1),
                     labels=c("0%", "25%", "50%", "75%", "100%"),
                     limits=c(-0.001, 1.001)) +
  scale_color_manual(name="",
                     values=c("P" = "blue3", "I" = "red2", "R" = "gold1")) +
  scale_fill_manual(name="",
                    values=c("P" = "blue3", "I" = "red2", "R" = "gold1")) +
  annotate("text", x=100, y=0.4, label="P", fontface="bold", size=14, color="white") +
  annotate("text", x=175, y=0.66, label="I", fontface="bold", size=14, color="white") +
  annotate("text", x=280, y=0.85, label="R", fontface="bold", size=14, color="white") +
  theme(axis.title.x = element_text(colour = 'black', size = 32, face = 'bold',
                                    margin=margin(t=0.2, unit = "cm")),
        axis.title.y = element_text(colour = 'black', size = 28, face = 'bold',
                                    margin=margin(r=0.2,l=1.5, unit = "cm")),
        axis.text.x = element_text(colour = 'black', size = 20, face = 'bold'),
        axis.text.y = element_text(colour = 'black', size = 20, face = 'bold'),
        axis.line = element_line(colour = 'black', size = 1.5, linetype = 'solid'),
        panel.background = element_rect(fill = "transparent", colour = NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(1.0, 0, 0.0, 0), "cm"),
        legend.position = "none",
        legend.title = element_text(colour="black", size=14, face="bold"),
        legend.text = element_text(colour="black", size=12, face="bold"),
        legend.background = element_rect(fill="white"),
        legend.key = element_rect(fill = "white"))


plP <- plP + annotation_custom(
  grob = textGrob(label = "B", hjust = 0,
                  gp = gpar(cex = 4, fontface="bold")),
  ymin = 1.07,
  ymax = 1.07,
  xmin = -70,
  xmax = -70)

pl2 <- ggplot_gtable(ggplot_build(plP))
pl2$layout$clip[pl2$layout$name == "panel"] <- "off"


pl <- grid.arrange(pl1, pl2, ncol=2, nrow=1,
                   layout_matrix= rbind(c(1, 2)),
                   heights=c(1), widths=c(0.9, 1),
                   bottom=textGrob(expression(paste("Planning Horizon (H)")),
                                   gp=gpar(fontsize=28,
                                           fontface="bold")))

ggsave(paste0(outputDir,"/figure3.pdf"), plot=pl,
       width=19.6, height=9)

