##
## Author......: Luis Gustavo Nardin
## Last Change.: 10/03/2016
##
library(data.table)
library(deSolve)
library(ggplot2)
library(grid)
library(gridExtra)


###############
## PATHS
###############
baseDir <- "."
disease1Data <- "../data/disease1"
disease2Data <- "../data/disease2"
outputDir <- "../figures"


###############
## FUNCTIONS
###############
source("calcSwitch.R")
source("SPIRmodel.R")


###############
## DISEASE 1 - INPUT PARAMETERS
###############
# Disease duration
duration <- 65

# R0
R0 <- 2

# gamma
gamma <- 1 / duration

# beta
betaS <- R0 / duration

# Infection probability (Susceptible)
bs <- 1 - exp(-betaS)

# Prophylactic protection
rho <- 0.0951

# Recover probability
g <- 1 - exp(-gamma)

# Discount factor (0 = No discount)
lambda <- 0

# Fear factor (1 = No fear)
kappa <- 1

# Decision frequency probability
delta <- 0.01

# Planning horizon
h <- 90

# Payoffs (S, P, I, R)
payoffs <- c(1, 0.95, 0.10, 0.95)

# Number of agents
N <- 100000

# I initialization
initI <- 100

# Initial values
yinit <- c(S = N - initI, P = 0, I = initI, R = 0)

# Length of simulation
times <- seq(1, 2100, 1)

h <- 1
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
data <- data.table(H=h, time=out$time, S=out$S, P=out$P, I=out$I, R=out$R)

h <- 30
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
data <- rbind(data, data.table(H=h, time=out$time, S=out$S, P=out$P, I=out$I, R=out$R))
isp <- data.table(H=h, i=iswitch[iswitch[,8] != 1,8])

h <- 45
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
data <- rbind(data, data.table(H=h, time=out$time, S=out$S, P=out$P, I=out$I, R=out$R))
isp <- rbind(isp, data.table(H=h, i=iswitch[iswitch[,8] != 1,8]))

h <- 90
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
data <- rbind(data, data.table(H=h, time=out$time, S=out$S, P=out$P, I=out$I, R=out$R))
isp <- rbind(isp, data.table(H=h, i=iswitch[iswitch[,8] != 1,8]))

pl <- ggplot(data, aes(x=time, y=((I / (S+P+I+R)) * 100),
                       color=as.factor(H),
                       size=as.factor(H))) +
  xlab("") +
  ylab(expression(paste("% Infectious (i)"))) +
  geom_line() +
  scale_color_manual(name = expression(paste("Planning\nHorizon (H)")),
                     values = c("grey60", "blue", "red", "green")) +
  scale_size_manual(name = expression(paste("Planning\nHorizon (H)")),
                    values = c(15, 10, 5, 2)) +
  scale_y_continuous(limits = c(0, 20),
                     breaks = c(0, 5, 10, 15, 20),
                     labels = c("0%", "5%", "10%", "15%", "20%")) +
  guides(color = guide_legend(override.aes=list(size=2))) +
  theme(axis.title.x = element_text(color='black', size=12, face='bold',
                                    margin=margin(t=0.5, unit = "cm")),
        axis.title.y = element_text(color='black', size=48, face='bold',
                                    margin=margin(r=0.5, unit = "cm")),
        axis.text.x = element_text(color='black', size=24, face='bold'),
        axis.text.y = element_text(color='black', size=24, face='bold'),
        axis.line.x = element_line(color='black', size=1, linetype='solid'),
        axis.line.y = element_line(color='black', size=1, linetype='solid'),
        panel.background = element_rect(fill = "transparent",color = NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(1, 0, 0, 0), "cm"),
        #legend.position = "none",
        legend.title = element_text(color="black", size=34, face="bold"),
        legend.text = element_text(color="black", size=28, face="bold"),
        legend.key = element_rect(fill="white"),
        legend.key.height=unit(2, "line"))

if(nrow(isp) > 0){
  pl <- pl + geom_hline(data=isp, aes(yintercept=i*100),
                        linetype="dashed", color=c("blue","red", "green"), size=1)
}

pl <- pl + annotation_custom(
  grob = textGrob(label = "A", hjust = 0,
                  gp = gpar(cex=5, fontface="bold")),
  ymin = 21,
  ymax = 21,
  xmin = -380,
  xmax = -380)

gth1 <- ggplot_gtable(ggplot_build(pl))
gth1$layout$clip[gth1$layout$name == "panel"] <- "off"


###############
## DISEASE 2 INPUT PARAMETERS
###############
# Disease duration
duration <- 8

# R0
R0 <- 2

# gamma
gamma <- 1 / duration

# beta
betaS <- R0 / duration

# Infection probability (Susceptible)
bs <- 1 - exp(-betaS)

# Prophylactic protection
rho <- 0.0099

# Recover probability
g <- 1 - exp(-gamma)

# Discount factor (0 = No discount)
lambda <- 0

# Fear factor (1 = No fear)
kappa <- 1

# Decision frequency
delta <- 0.01

# Planning horizon
h <- 30

# Payoffs (S, P, I, R)
payoffs <- c(1, 0.95, 0.60, 1)

# Number of agents
N <- 100000

# I initialization
initI <- 100

# Initial values
yinit <- c(S = N - initI, P = 0, I = initI, R = 0)

# Length of simulation
times <- seq(1, 250, 1)

h <- 1
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
data <- data.table(H=h, time=out$time, S=out$S, P=out$P, I=out$I, R=out$R)

h <- 30
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
data <- rbind(data, data.table(H=h, time=out$time, S=out$S, P=out$P, I=out$I, R=out$R))
isp <- data.table(H=h, i=iswitch[iswitch[,8] != 1,8])

h <- 45
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
data <- rbind(data, data.table(H=h, time=out$time, S=out$S, P=out$P, I=out$I, R=out$R))
isp <- rbind(isp, data.table(H=h, i=iswitch[iswitch[,8] != 1,8]))

h <- 90
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
data <- rbind(data, data.table(H=h, time=out$time, S=out$S, P=out$P, I=out$I, R=out$R))

pl <- ggplot(data, aes(x=time, y=((I / (S+P+I+R)) * 100),
                       colour=as.factor(H),
                       size=as.factor(H))) +
  xlab("") +
  ylab("") +
  geom_line() +
  scale_colour_manual(name = expression(paste("Planning\nHorizon (H)")),
                      values = c("grey60", "blue", "red", "green")) +
  scale_size_manual(name = expression(paste("Planning\nHorizon (H)")),
                    values = c(15, 10, 5, 2)) +
  scale_y_continuous(limits = c(0, 20),
                     breaks = c(0, 5, 10, 15, 20),
                     labels = c("0%", "5%", "10%", "15%", "20%")) +
  guides(colour=guide_legend(override.aes=list(size=2))) +
  theme(axis.title.x = element_text(colour='black', size=12, face='bold',
                                    margin=margin(t=0.5, unit = "cm")),
        axis.title.y = element_text(colour='black', size=48, face='bold',
                                    margin=margin(r=0.5, unit = "cm")),
        axis.text.x = element_text(colour='black', size=24, face='bold'),
        axis.text.y = element_text(colour='black', size=24, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill = "transparent",colour = NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(1, 0, 0, 0), "cm"),
        #legend.position = "none",
        legend.title = element_text(colour="black", size=34, face="bold"),
        legend.text = element_text(colour="black", size=28, face="bold"),
        legend.key = element_rect(fill="white"),
        legend.key.height=unit(2, "line"))

if(nrow(isp) > 0){
  pl <- pl + geom_hline(data=isp, aes(yintercept=i*100),
                        linetype="dashed", color=c("blue","red"), size=1)
}

pl <- pl + annotation_custom(
  grob = textGrob(label = "B", hjust = 0,
                  gp = gpar(cex=5, fontface="bold")),
  ymin = 21,
  ymax = 21,
  xmin = -47,
  xmax = -47)

gth2 <- ggplot_gtable(ggplot_build(pl))
gth2$layout$clip[gth2$layout$name == "panel"] <- "off"

plot <- grid.arrange(gth1, gth2, ncol=2, nrow=1,
                     layout_matrix= rbind(c(1, 2)),
                     heights=c(1), widths=c(0.5, 0.5),
                     bottom=textGrob(expression(paste("Time (t)")),
                                     gp=gpar(fontsize=48,
                                             fontface="bold")))

ggsave(paste0(outputDir,"/figure5.pdf"), plot=plot,
       width=80, height=25, units="cm")
