##
## Author......: Luis Gustavo Nardin
## Last Change.: 09/25/2016
##
library(colorRamps)
library(data.table)
library(ggplot2)
library(grid)
library(gridExtra)


###############
## PATHS
###############
baseDir <- "."
disease2Data <- "../data/disease2"
outputDir <- "../figures"


###############
## FUNCTIONS
###############
source("calcSwitch.R")
source("calcUtilities.R")
source("SPIRmodel.R")


###############
## DISEASE 2 INPUT PARAMETERS
###############
# Disease duration
duration <- 8

# R0
R0 <- 2

# gamma
gamma <- 1 / duration

# beta
betaS <- R0 / duration

# Infection probability (Susceptible)
bs <- 1 - exp(-betaS)

# Prophylactic protection
rho <- 0.01

# Recover probability
g <- 1 - exp(-gamma)

# Discount factor (0 = No discount)
lambda <- 0

# Fear factor (1 = No fear)
kappa <- 1

# Decision frequency
delta <- 0

# Planning horizon
h <- 30


# Payoffs (S, P, I, R)
payoffs <- c(1, 0.95, 0.60, 1)

# Initial values
yinit <- c(S = 100000 - 1, P = 0, I = 1, R = 0)

# Length of simulation
times <- seq(1, 250, 1)


###############
## HEAT MAP
###############
filename <- "disease2-1"
data <- data.table(read.table(paste0(disease2Data, "/", filename,".csv"),
                              sep=";", header=TRUE))

maxh <- 365
pData <- data[which((h <= maxh))]

pl <- ggplot(pData[which((n == 0) & (i < 1))],
             aes(x=h, y=(1 - rho) * 100, fill=(i * 100))) +
  xlab(expression(paste("Planning Horizon (H)"))) +
  ylab(expression(paste("% Protection"))) +
  xlim(0, maxh + 20) +
  geom_raster(interpolate=TRUE, stat="identity") +
  geom_contour(data=pData[which(n == 2)],
               aes(x=h, y=(1 - rho) * 100, z=pI),
               breaks=c(0.3, 0.5, 0.7, 0.9),
               color="black",
               linetype="solid",
               size=0.5) +
  geom_line(data=pData[which(n == 2)],
            alpha=0.05,
            size=1) +
  geom_segment(aes(x = 40, y = 77, xend = 50, yend = 75),
               color="black", show.legend=FALSE) +
  annotate("text", x=150, y=50, label="A", fontface="italic", size=7) +
  annotate("text", x=25, y=90, label="B", fontface="italic", size=7) +
  annotate("text", x=55, y=75, label="C", fontface="italic", size=7) +
  scale_y_continuous(limits = c(0, 100),
                     breaks = c(0, 25, 50, 75, 100),
                     labels = c("0%", "25%", "50%", "75%", "100%")) +
  scale_fill_gradientn(name = expression(paste("% Infectious (i)")),
                       limits = c(0, 100),
                       values = c(0.0, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0),
                       colors = blue2green2red(50),
                       labels = c("0%", "25%", "50%", "75%", "100%")) +
  theme(axis.title.x = element_text(color='black', size=20, face='bold',
                                    margin=margin(t=0.2, unit = "cm")),
        axis.title.y = element_text(color='black', size=20, face='bold',
                                    margin=margin(r=0.5, unit = "cm")),
        axis.text.x = element_text(color='black', size=16, face='bold'),
        axis.text.y = element_text(color='black', size=16, face='bold'),
        axis.line.x = element_line(color='black', size=1, linetype='solid'),
        axis.line.y = element_line(color='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", color=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(0.5, 0, 0.5, 0), "cm"),
        #legend.position = "none",
        legend.title = element_text(color="black", size=18, face="bold"),
        legend.text = element_text(color="black", size=18, face="bold"),
        legend.key = element_rect(fill = "white"))

pl <- pl + annotation_custom(
  grob = textGrob(label = "D", hjust = 0,
                  gp = gpar(cex=2.5, fontface="bold")),
  ymin = 108,
  ymax = 108,
  xmin = -55,
  xmax = -55)

gt <- ggplot_gtable(ggplot_build(pl))
gt$layout$clip[gt$layout$name == "panel"] <- "off"


###############
## NO SWITCHING
###############
rho <- 0.5
h <- 1

data <- data.table(calc_utilities(h, bs, rho, g, lambda, kappa, payoffs))

data$state <- factor(data$state, levels=rev(levels(as.factor(data$state))))

ymax <- max(as.numeric(as.character(data[which(i > 0)]$U)))

pu1 <- ggplot(data[which(i > 0)], aes(x=as.numeric(as.character(i)) * 100,
                                      y=as.numeric(as.character(U)),
                                      group=state,
                                      color=state,
                                      linetype=state)) +
  xlab("") + ylab(expression(paste("Utility"))) +
  geom_line(size=0.9) +
  scale_x_continuous(breaks = c(0, 50, 100),
                     labels = c("0%", "50%", "100%"),
                     limits = c(0, 110)) +
  scale_linetype_manual(name = "",
                        values = c("solid", "dashed"),
                        labels = c("Susceptible", "Prophylactic")) +
  scale_color_manual(name ="",
                     values = c("black", "black"),
                     labels = c("Susceptible", "Prophylactic")) +
  theme(axis.title.x = element_text(colour = 'black', size = 20, face = 'bold',
                                    margin = margin(t=0.2, unit = "cm")),
        axis.title.y = element_text(colour = 'black', size = 20, face = 'bold',
                                    margin = margin(r=0.4, unit = "cm")),
        axis.text.x = element_text(colour = 'black', size = 14, face = 'bold'),
        axis.text.y = element_blank(),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill = "transparent", colour = NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(0.5, 0, 0.5, 0), "cm"),
        legend.position = "none",
        legend.title = element_text(colour="black", size=20, face="bold"),
        legend.text = element_text(colour="black", size=20, face="bold"),
        legend.key = element_rect(fill = "white"))

pu1 <- pu1 + annotation_custom(
  grob = textGrob(label = "A", hjust = 0,
                  gp = gpar(cex=2.5, fontface="bold")),
  ymin = ymax + 0.005,
  ymax = ymax + 0.005,
  xmin = -20,
  xmax = -20)

gt1 <- ggplot_gtable(ggplot_build(pu1))
gt1$layout$clip[gt1$layout$name == "panel"] <- "off"


###############
## ONE SWITCHING POINT
###############
rho <- 0.5
h <- 10

data <- data.table(calc_utilities(h, bs, rho, g, lambda, kappa, payoffs))

data$state <- factor(data$state, levels=rev(levels(as.factor(data$state))))

ymax <- max(as.numeric(as.character(data[which(i > 0)]$U)))

xint <- data.table(calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs))

pu2 <- ggplot(data[which(i > 0)], aes(x=as.numeric(as.character(i)) * 100,
                                      y=as.numeric(as.character(U)),
                                      group=state,
                                      color=state,
                                      linetype=state)) +
  xlab(expression(paste("% Infectious"))) + ylab("") +
  geom_vline(xintercept=xint[which(n == 1)]$pI * 100,
             linetype="dotted", size=1) +
  geom_line(size=0.9) +
  scale_x_continuous(breaks = c(0, 50, 100),
                     labels = c("0%", "50%", "100%"),
                     limits = c(0, 110)) +
  scale_linetype_manual(name = "",
                        values = c("solid", "dashed"),
                        labels = c("Susceptible", "Prophylactic")) +
  scale_color_manual(name="",
                     values=c("black", "black"),
                     labels=c("Susceptible", "Prophylactic")) +
  theme(axis.title.x = element_text(colour = 'black', size = 20, face = 'bold',
                                    margin = margin(t=0.2, unit = "cm")),
        axis.title.y = element_text(colour = 'black', size = 20, face = 'bold',
                                    margin = margin(r=0.4, unit = "cm")),
        axis.text.x = element_text(colour = 'black', size = 14, face = 'bold'),
        axis.text.y = element_blank(),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill = "transparent", colour = NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(0.5, 0, 0.5, 0), "cm"),
        legend.position = "none",
        legend.title = element_text(colour="black", size=18, face="bold"),
        legend.text = element_text(colour="black", size=14, face="bold"),
        legend.key = element_rect(fill = "white"))

pu2 <- pu2 + annotation_custom(
  grob = textGrob(label = "B", hjust = 0,
                  gp = gpar(cex=2.5, fontface="bold")),
  ymin = ymax + 0.10,
  ymax = ymax + 0.10,
  xmin = -20,
  xmax = -20)

gt2 <- ggplot_gtable(ggplot_build(pu2))
gt2$layout$clip[gt2$layout$name == "panel"] <- "off"

###############
## TWO SWITCHING POINTS
###############
rho <- 0.20
h <- 37

data <- data.table(calc_utilities(h, bs, rho, g, lambda, kappa, payoffs))

data$state <- factor(data$state, levels=rev(levels(as.factor(data$state))))

ymax <- max(as.numeric(as.character(data[which(i > 0)]$U)))

xint <- data.table(calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs))

pu3 <- ggplot(data[which(i > 0)], aes(x=as.numeric(as.character(i)) * 100,
                                      y=as.numeric(as.character(U)),
                                      group=state,
                                      color=state,
                                      linetype=state)) +
  xlab("") + ylab("") +
  geom_vline(xintercept=xint[which(n == 1)]$pI * 100,
             linetype="dotted", size=1) +
  geom_vline(xintercept=xint[which(n == 2)]$pI * 100,
             linetype="dotted", size=1) +
  geom_line(size=0.9) +
  geom_line(size=0.9) +
  scale_x_continuous(breaks = c(0, 50, 100),
                     labels = c("0%", "50%", "100%"),
                     limits = c(0, 110)) +
  scale_linetype_manual(name = "",
                        values = c("solid", "dashed"),
                        labels = c(expression(paste("Susceptible")),
                                   expression(paste("Prophylactic")))) +
  scale_color_manual(name = "",
                     values = c("black", "black"),
                     labels = c(expression(paste("Susceptible")),
                                expression(paste("Prophylactic")))) +
  theme(axis.title.x = element_text(colour = 'black', size = 20, face = 'bold',
                                    margin = margin(t=0.2, unit = "cm")),
        axis.title.y = element_text(colour = 'black', size = 20, face = 'bold',
                                    margin = margin(r=0.4, unit = "cm")),
        axis.text.x = element_text(colour = 'black', size = 14, face = 'bold'),
        axis.text.y = element_blank(),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill = "transparent", colour = NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(0.5, 0, 0.5, 0), "cm"),
        #legend.position = "none",
        legend.title = element_text(colour="black", size=18, face="bold"),
        legend.text = element_text(colour="black", size=14, face="bold"),
        legend.key = element_rect(fill = "white"))

pu3 <- pu3 + annotation_custom(
  grob = textGrob(label = "C", hjust = 0,
                  gp = gpar(cex=2.5, fontface="bold")),
  ymin = ymax + 0.15,
  ymax = ymax + 0.15,
  xmin = -20,
  xmax = -20)

gt3 <- ggplot_gtable(ggplot_build(pu3))
gt3$layout$clip[gt3$layout$name == "panel"] <- "off"

plot <- grid.arrange(gt1, gt2, gt3, gt, ncol=3, nrow=2,
                     layout_matrix= rbind(c(1,2,3),
                                          c(4,4,4)),
                     heights=c(1.5,3), widths=c(0.65,0.65,1))

ggsave(paste0(outputDir, "/figureS1.pdf"), plot=plot,
       width=14.69, height=9.00)
