##
## Author......: Luis Gustavo Nardin
## Last Change.: 09/25/2016
##
library(colorRamps)
library(data.table)
library(deSolve)
library(ggplot2)
library(grid)
library(gridExtra)


###############
## PATHS
###############
baseDir <- "."
disease2Data <- "../data/disease2"
outputDir <- "../figures"


###############
## FUNCTIONS
###############
source("calcSwitch.R")
source("calcUtilities.R")
source("SPIRmodel.R")


###############
## DISEASE 2 INPUT PARAMETERS
###############
# Disease duration
duration <- 8

# R0
R0 <- 2

# gamma
gamma <- 1 / duration

# beta
betaS <- R0 / duration

# Infection probability (Susceptible)
bs <- 1 - exp(-betaS)

# Prophylactic protection
rho <- 0.01

# Recover probability
g <- 1 - exp(-gamma)

# Discount factor (0 = No discount)
lambda <- 0

# Fear factor (1 = No fear)
kappa <- 1

# Decision frequency
delta <- 0

# Planning horizon
h <- 30


# Payoffs (S, P, I, R)
payoffs <- c(1, 0.95, 0.60, 1)

# Initial values
yinit <- c(S = 100000 - 1, P = 0, I = 1, R = 0)

# Length of simulation
times <- seq(1, 250, 1)


###############
## FEAR
###############
filename <- "disease2-1"
data <- data.table(read.table(paste0(disease2Data, "/", filename,".csv"),
                              sep=";", header=TRUE))

maxh <- 365
pData <- data[which((h <= maxh))]

pf1 <- ggplot(pData[which((n == 0) & (i < 1))],
              aes(x=h, y=(1 - rho) * 100, fill=(i * 100))) +
  xlab("") +
  ylab(expression(paste("% Protection"))) +
  xlim(0, maxh + 60) +
  geom_raster(interpolate=TRUE, stat="identity") +
  geom_contour(data=pData[which(n == 2)],
               aes(x=h, y=(1 - rho) * 100, z=pI),
               breaks=c(0.3, 0.5, 0.7, 0.9),
               color="black",
               linetype="solid",
               size=0.5) +
  geom_line(data=pData[which(n == 2)],
            alpha=0.05,
            size=1) +
  annotate("text", x=190, y=65, label="kappa == 1.0", fontface="bold", size=7, parse=TRUE) +
  scale_y_continuous(limits = c(0, 100),
                     breaks = c(0, 25, 50, 75, 100),
                     labels = c("0%", "25%", "50%", "75%", "100%")) +
  scale_fill_gradientn(name = expression(paste("% Infectious (i)")),
                       limits = c(0, 100),
                       values = c(0.0, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0),
                       colors = blue2green2red(50),
                       labels = c("0%", "25%", "50%", "75%", "100%")) +
  theme(axis.title.x = element_text(colour = 'black', size = 20, face = 'bold',
                                    margin = margin(t=0.2, unit = "cm")),
        axis.title.y = element_text(colour = 'black', size = 20, face = 'bold',
                                    margin = margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour='black', size=16, face='bold'),
        axis.text.y = element_text(colour='black', size=16, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", colour=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(1, 0, 0, 0), "cm"),
        legend.position = "none",
        legend.title = element_text(colour="black", size=18, face="bold"),
        legend.text = element_text(colour="black", size=14, face="bold"),
        legend.key = element_rect(fill = "white"))

pf1 <- pf1 + annotation_custom(
  grob = textGrob(label = "A", hjust = 0,
                  gp = gpar(cex=2.5, fontface="bold")),
  ymin = 110,
  ymax = 110,
  xmin = -90,
  xmax = -90)

gtf1 <- ggplot_gtable(ggplot_build(pf1))
gtf1$layout$clip[gtf1$layout$name == "panel"] <- "off"


filename <- "disease2-k1.5"
data <- data.table(read.table(paste0(disease2Data, "/", filename,".csv"),
                              sep=";", header=TRUE))

maxh <- 365
pData <- data[which((h <= maxh))]

pf2 <- ggplot(pData[which((n == 0) & (i < 1))],
              aes(x=h, y=(1 - rho) * 100, fill=(i * 100))) +
  xlab("") +
  ylab("") +
  xlim(0, maxh + 60) +
  geom_raster(interpolate=TRUE, stat="identity") +
  geom_contour(data=pData[which(n == 2)],
               aes(x=h, y=(1 - rho) * 100, z=pI),
               breaks=c(0.3, 0.5, 0.7, 0.9),
               color="black",
               linetype="solid",
               size=0.5) +
  geom_line(data=pData[which(n == 2)],
            alpha=0.05,
            size=1) +
  annotate("text", x=190, y=65, label="kappa == 1.5", fontface="bold", size=7, parse=TRUE) +
  scale_y_continuous(limits = c(0, 100),
                     breaks = c(0, 25, 50, 75, 100),
                     labels = c("0%", "25%", "50%", "75%", "100%")) +
  scale_fill_gradientn(name = expression(paste("% Infectious (i)")),
                       limits = c(0, 100),
                       values = c(0.0, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0),
                       colors = blue2green2red(50),
                       labels = c("0%", "25%", "50%", "75%", "100%")) +
  theme(axis.title.x = element_text(colour='black', size=20, face='bold',
                                    margin=margin(t=0.2, unit = "cm")),
        axis.title.y = element_text(colour='black', size=20, face='bold',
                                    margin=margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour='black', size=16, face='bold'),
        axis.text.y = element_text(colour='black', size=16, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", colour=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(1, 0, 0, 0), "cm"),
        #legend.position = "none",
        legend.title = element_text(colour="black", size=18, face="bold"),
        legend.text = element_text(colour="black", size=14, face="bold"),
        legend.key = element_rect(fill = "white"))

pf2 <- pf2 + annotation_custom(
  grob = textGrob(label = "B", hjust = 0,
                  gp = gpar(cex=2.5, fontface="bold")),
  ymin = 110,
  ymax = 110,
  xmin = -90,
  xmax = -90)

gtf2 <- ggplot_gtable(ggplot_build(pf2))
gtf2$layout$clip[gtf2$layout$name == "panel"] <- "off"

gtf <- grid.arrange(gtf1, gtf2, ncol=2, nrow=1,
                    layout_matrix= rbind(c(1, 2)),
                    heights=c(1), widths=c(0.75, 1),
                    bottom=textGrob(expression(paste("Planning Horizon (H)")),
                                    gp=gpar(fontsize=12,
                                            fontface="bold")))

rho <- 0.1
h <- 30
delta <- 0.01
kappa <- 1
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
data <- data.table(K=kappa, time=out$time, S=out$S, P=out$P, I=out$I, R=out$R)
isp <- data.table(i=iswitch[iswitch[,8] != 1,8])

h <- 30
delta <- 0.1
kappa <- 1.2
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
data <- rbind(data, data.table(K=kappa, time=out$time, S=out$S, P=out$P, I=out$I, R=out$R))
isp <- rbind(isp, data.table(i=iswitch[iswitch[,8] != 1,8]))

h <- 30
delta <- 0.1
kappa <- 1.5
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
data <- rbind(data, data.table(K=kappa, time=out$time, S=out$S, P=out$P, I=out$I, R=out$R))
isp <- rbind(isp, data.table(i=iswitch[iswitch[,8] != 1,8]))


pl <- ggplot(data, aes(x=time, y=((I / (S+P+I+R)) * 100),
                       colour=as.factor(K),
                       size=as.factor(K))) +
  xlab(expression(paste("Time (t)"))) +
  ylab(expression(paste("% Infectious (i)"))) +
  geom_line() +
  scale_colour_manual(name = expression(paste("Distortion of\nPerception (",kappa,")")),
                      values = c("grey60", "blue", "red"),
                      labels = c("1.0", "1.2", "1.5")) +
  scale_size_manual(name = expression(paste("Distortion of\nPerception (",kappa,")")),
                    values = c(3, 1.5, 0.75),
                    labels = c("1.0", "1.2", "1.5")) +
  scale_y_continuous(limits = c(0, 17),
                     breaks = c(0, 5, 10, 15),
                     labels = c("0%", "5%", "10%", "15%")) +
  guides(colour = guide_legend(override.aes = list(size=1))) +
  theme(axis.title.x = element_text(colour='black', size=20, face='bold',
                                    margin=margin(t=0.2, unit = "cm")),
        axis.title.y = element_text(colour='black', size=20, face='bold',
                                    margin=margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour='black', size=16, face='bold'),
        axis.text.y = element_text(colour='black', size=16, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", colour=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(1, 0, 1, 0), "cm"),
        #legend.position = "none",
        legend.title = element_text(colour="black", size=18, face="bold"),
        legend.text = element_text(colour="black", size=14, face="bold"),
        legend.key = element_rect(fill = "white"))

if(nrow(isp) > 0){
  pl <- pl + geom_hline(data=isp, aes(yintercept=i*100),
                        linetype="dashed", color=c("grey60", "blue", "red"), size=c(1,1,1))
}

pl <- pl + annotation_custom(
  grob = textGrob(label = "C", hjust = 0,
                  gp = gpar(cex=2.5, fontface="bold")),
  ymin = 19,
  ymax = 19,
  xmin = -28,
  xmax = -28)

gt <- ggplot_gtable(ggplot_build(pl))
gt$layout$clip[gt$layout$name == "panel"] <- "off"

plot <- grid.arrange(gtf, gt, ncol=1, nrow=2,
                    layout_matrix= rbind(c(1),c(2)),
                    heights=c(1,1), widths=c(1))

ggsave(paste0(outputDir, "/figureS2.pdf"), plot=plot,
       width=14.69, height=9.00)
