##
## Disease 1
## ABM - ODE Comparison Plot
##
## Author......: Luis Gustavo Nardin
## Last Change.: 10/03/2016
##
library(colorRamps)
library(data.table)
library(deSolve)
library(ggplot2)
library(grid)
library(gridExtra)
library(gtable)


###############
## PATHS
###############
baseDir <- "."
disease1Data <- "../data/disease1"
outputDir <- "../figures"


###############
## FUNCTIONS
###############
source("calcSwitch.R")
source("SPIRmodel.R")


###############
## DISEASE 1 - INPUT PARAMETERS
###############
# Disease duration
duration <- 65

# R0
R0 <- 2

# gamma
gamma <- 1 / duration

# beta
betaS <- R0 / duration

# Infection probability (Susceptible)
bs <- 1 - exp(-betaS)

# Prophylactic protection
rho <- 0.0951

# Recover probability
g <- 1 - exp(-gamma)

# Discount factor (0 = No discount)
lambda <- 0

# Fear factor (1 = No fear)
kappa <- 1

# Decision frequency probability
delta <- 0.01

# Planning horizon
h <- 90

# Payoffs (S, P, I, R)
payoffs <- c(1.00, 0.95, 0.10, 0.95)

# Number of agents
N <- 100000

# I initialization
initI <- 100

# Initial values
yinit <- c(S = N - initI, P = 0, I = initI, R = 0)

# Length of simulation
times <- seq(1, 2100, 1)


###############
## ODE
###############
iswitch <- calc_iswitch(h, bs, rho, g, lambda, kappa, payoffs)
pars <- list(R0, duration, gamma, betaS, delta, iswitch)
out <- as.data.frame(lsoda(yinit, times, SPIRmodel, pars, rtol=1e-3, atol=1e-3))
dataODE <- data.table(simulation=0, H=h, replication=0, time=out$time,
                      S=out$S, P=out$P, I=out$I, R=out$R)


###############
## ABM
###############
dataABM <- fread(paste0(disease1Data, "/disease1-abm.csv"),
                 sep=";", header=TRUE)

dataABM <- data.table(simulation=1, dataABM)


###############
## PLOT
###############
pl <- ggplot() +
  geom_line(data=dataABM,
            aes(x=time, y=(I / (S + P + I + R) * 100),
                group=replication),
            color="grey60", size=0.5) +
  geom_line(data=dataODE,
            aes(x=time, y=(I / (S + P + I + R) * 100)),
            color="black", size=1.5) +
  xlab("Time") +
  ylab(expression(paste("% Infectious (i)"))) +
  scale_y_continuous(limits = c(0, 10),
                     breaks = c(0, 5, 10),
                     labels = c("0%", "5%", "10%")) +
  guides(color = guide_legend(override.aes=list(size=2))) +
  theme(axis.title.x = element_text(color='black', size=44, face='bold',
                                    margin=margin(t=0.5, unit = "cm")),
        axis.title.y = element_text(color='black', size=48, face='bold',
                                    margin=margin(r=0.5, unit = "cm")),
        axis.text.x = element_text(color='black', size=24, face='bold'),
        axis.text.y = element_text(color='black', size=24, face='bold'),
        axis.line.x = element_line(color='black', size=1, linetype='solid'),
        axis.line.y = element_line(color='black', size=1, linetype='solid'),
        panel.background = element_rect(fill = "transparent",color = NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        plot.margin = unit(c(1, 0, 0, 0), "cm"),
        legend.position = "none",
        legend.title = element_text(color="black", size=34, face="bold"),
        legend.text = element_text(color="black", size=28, face="bold"),
        legend.key = element_rect(fill = "white"),
        legend.key.height=unit(1.5,"line"))

ggsave(paste0(outputDir,"/figureS3.pdf"), plot=pl,
       width=40, height=25, units="cm")


###############
## STATISTICAL COMPARISON
###############
odeI <- with(dataODE, I / (S+P+I+R))

for(rep in unique(dataABM$replication)){
  abmI <- with(dataABM[which(replication == rep)], I / (S+P+I+R))
  w <- wilcox.test(odeI, abmI, paired=FALSE, alternative="two.sided")
  
  print(w$p.value)
}